--
-- Specialization for Animations
-- This script can be used to play animations that use the AnimatedVehicle specializations
--
-- @author:		Xentro (www.fs-uk.info)(Marcus@Xentro.se)
-- @version:	v1.3
-- @date:		2013-02-01
-- @history:	v1.0 - inital implementation
-- 				v1.1 - added save/load function
-- 				v1.2 - added the feature to remove inputbinding
--				v1.3 - minor cleaning 
--
-- <standardAnimations>
--	   <part name="animationName" inputName="BUTTONNAME" helpTextNamePos="110n_help_text_open" helpTextNameNeg="110n_help_text_close" />
-- </standardAnimations>

PlayStandAnim = {};

function PlayStandAnim.prerequisitesPresent(specializations)
    if not SpecializationUtil.hasSpecialization(AnimatedVehicle, specializations) then print("Warning: Specialization PlayStandAnim needs the specialization AnimatedVehicle"); end;
	
	return SpecializationUtil.hasSpecialization(AnimatedVehicle, specializations);
end;

function PlayStandAnim:load(xmlFile)
	self.setPlayAnim = SpecializationUtil.callSpecializationsFunction("setPlayAnim");

	self.PAs = {};
	local i = 0;
	while true do
		local baseName = string.format("vehicle.standardAnimations.part(%d)", i);
		if not hasXMLProperty(xmlFile, baseName) then break; end;
		
		local PA = {};
		local name = getXMLString(xmlFile, baseName .. "#name");
		local inputName = getXMLString(xmlFile, baseName .. "#inputName");
		if name == nil or self.animations[name] == nil then
			print("Error: Can't find the animation (" .. tostring(name) .. ") for standardAnimations part number " .. i);
			break;
		end;
		
		PA.aName = name;
		PA.b = false;
		
		if inputName ~= nil then
			if InputBinding[inputName] ~= nil then
				PA.input = inputName;
				
				PA.htp = getXMLString(xmlFile, baseName .. "#helpTextNamePos");
				PA.htn = getXMLString(xmlFile, baseName .. "#helpTextNameNeg");
			else
				print("Error: Can't find the InputBinding (" .. tostring(inputName) .. ") for standardAnimations part number " .. i);
				break;
			end;
		end;
			
		
		table.insert(self.PAs, PA);
		i = i + 1;
	end;
end;

function PlayStandAnim:delete()
end;

function PlayStandAnim:mouseEvent(posX, posY, isDown, isUp, button)
end;

function PlayStandAnim:keyEvent(unicode, sym, modifier, isDown)
end;

function PlayStandAnim:readStream(streamId, connection)
	for k,v in ipairs(self.PAs) do
		local state = streamReadBool(streamId);
		self:setPlayAnim(k, state, true);
	end;
end;
	
function PlayStandAnim:writeStream(streamId, connection)
	for k,v in ipairs(self.PAs) do
		streamWriteBool(streamId, v.b);
	end;
end;

function PlayStandAnim:loadFromAttributesAndNodes(xmlFile, key, resetVehicles)
	if not resetVehicles then		
		local numPlayStandAnim = getXMLInt(xmlFile, key .. "#numPlayStandAnim");
		if numPlayStandAnim ~= nil then
			for i=1, numPlayStandAnim do
				local bKey = key..string.format(".PlayStandAnim(%d)", i - 1);
				local state = Utils.getNoNil(getXMLBool(xmlFile, bKey .. "#b"), false);
				
				if state ~= nil then
					self:setPlayAnim(i, state);
				end;
			end;
		end;
	end; 
	return BaseMission.VEHICLE_LOAD_OK;
end;

function PlayStandAnim:getSaveAttributesAndNodes(nodeIdent)
	local attributes = 'numPlayStandAnim="' .. table.getn(self.PAs) .. '"';
	local nodes = "";
	local numAnim = 0;
	for i=1, table.getn(self.PAs) do
		local b = self.PAs[i];
		if numAnim > 0 then
			nodes = nodes .. "\n";
		end;
		nodes = nodes .. nodeIdent .. '<PlayStandAnim b="' .. tostring(b.b) .. '"';
		nodes = nodes .. ' />';
		numAnim = numAnim + 1;
	end;
	
    return attributes, nodes;
end;

function PlayStandAnim:update(dt)
	if self:getIsActive() then
		if self:getIsActiveForInput() then
			for k, v in ipairs(self.PAs) do
				if v.input ~= nil then
					if InputBinding.hasEvent(InputBinding[v.input]) then
						self:setPlayAnim(k, not v.b); 
					end;
				end;
			end;
		end;
	end;
end;

function PlayStandAnim:draw()
	if g_currentMission.showHelpText then
		for _, v in ipairs(self.PAs) do
			if v.input ~= nil then
				if g_i18n:hasText(v.htp) and g_i18n:hasText(v.htp) then
					if v.b then
						g_currentMission:addHelpButtonText(g_i18n:getText(v.htn), InputBinding[v.input]);
					else
						g_currentMission:addHelpButtonText(g_i18n:getText(v.htp), InputBinding[v.input]);
					end;
				else
					if g_i18n:hasText(v.htp) then
						g_currentMission:addHelpButtonText(g_i18n:getText(v.htp), InputBinding[v.input]);
					elseif not g_i18n:hasText(v.htp) and g_i18n:hasText(v.htn) then
						g_currentMission:addHelpButtonText(g_i18n:getText(v.htn), InputBinding[v.input]);
					end;
				end;
			end;
		end;
	end;
end;

function PlayStandAnim:setPlayAnim(id, state, noEventSend)
	local v = self.PAs[id];
	if v ~= nil and state ~= nil then
		if state then
			if v.aName ~= nil and self.playAnimation ~= nil then
				self:playAnimation(v.aName, 1, nil, true);
				v.b = state;
			end;
		else
			if v.aName ~= nil and self.playAnimation ~= nil then
				self:playAnimation(v.aName, -1, nil, true);
				v.b = state;
			end;
		end;
		setPlayAnimeEvent.sendEvent(self, id, state, noEventSend);
	end;
end;

setPlayAnimeEvent = {};
setPlayAnimeEvent_mt = Class(setPlayAnimeEvent, Event);
InitEventClass(setPlayAnimeEvent, "setPlayAnimeEvent");

function setPlayAnimeEvent:emptyNew()
    local self = Event:new(setPlayAnimeEvent_mt);
    self.className="setPlayAnimeEvent";
    return self;
end;

function setPlayAnimeEvent:new(object, id, state)
	local self = setPlayAnimeEvent:emptyNew()
	self.object = object;
	self.id = id;
	self.state = state;
	return self;
end;

function setPlayAnimeEvent:readStream(streamId, connection)
    self.object = networkGetObject(streamReadInt32(streamId));
	self.id  = streamReadInt8(streamId);
    self.state = streamReadBool(streamId);
    self:run(connection);
end;

function setPlayAnimeEvent:writeStream(streamId, connection)
    streamWriteInt32(streamId, networkGetObjectId(self.object));
	streamWriteInt8(streamId, self.id);
	streamWriteBool(streamId, self.state);
end;

function setPlayAnimeEvent:run(connection)
	self.object:setPlayAnim(self.id,self.state, true);
	if not connection:getIsServer() then
		g_server:broadcastEvent(setPlayAnimeEvent:new(self.object, self.id, self.state), nil, connection, self.object);
	end;	
end;

function setPlayAnimeEvent.sendEvent(vehicle, id, state, noEventSend)
	if noEventSend == nil or noEventSend == false then
		if g_server ~= nil then
			g_server:broadcastEvent(setPlayAnimeEvent:new(vehicle, id, state), nil, nil, vehicle);
		else
			g_client:getServerConnection():sendEvent(setPlayAnimeEvent:new(vehicle, id, state));
		end;
	end;
end;